"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FormServiceDynamic = void 0;
const core_1 = require("@angular/core");
const forms_1 = require("@angular/forms");
let FormServiceDynamic = exports.FormServiceDynamic = class FormServiceDynamic {
    constructor(formBuilder) {
        this.formBuilder = formBuilder;
    }
    createForm(model, options) {
        const formGroup = {};
        for (const field in model) {
            if (model.hasOwnProperty(field)) {
                const fieldConfig = options && options[field] ? options[field] : {};
                formGroup[field] = this.createControl(model[field], fieldConfig);
            }
        }
        return this.formBuilder.group(formGroup);
    }
    createControl(value, config) {
        const formControl = this.formBuilder.control({ value: value, disabled: !config.enabled }, this.normalizeValidations(config.validations), this.normalizeAsyncValidations(config.asyncValidations));
        if (config.customValidation) {
            formControl.setValidators([config.customValidation]);
        }
        if (config.asyncCustomValidation) {
            formControl.setAsyncValidators([config.asyncCustomValidation]);
        }
        if (config.requiredIf) {
            formControl.setValidators([this.conditionallyRequiredValidator(config)]);
        }
        return formControl;
    }
    conditionallyRequiredValidator(config) {
        return (control) => {
            if (this.isConditionallyRequired(config)) {
                if (forms_1.Validators.required(control)) {
                    return null;
                }
                else {
                    return { required: true };
                }
            }
            return null;
        };
    }
    isConditionallyRequired(config) {
        return config.requiredIf && config.requiredIf.condition ? config.requiredIf.condition() : false;
    }
    normalizeValidations(validations) {
        if (!validations) {
            return null;
        }
        const validatorArray = [];
        for (const key in validations) {
            if (validations.hasOwnProperty(key) && forms_1.Validators.hasOwnProperty(key)) {
                validatorArray.push(forms_1.Validators[key](validations[key]));
            }
        }
        return validatorArray.length > 0 ? forms_1.Validators.compose(validatorArray) : null;
    }
    normalizeAsyncValidations(asyncValidations) {
        if (!asyncValidations) {
            return null;
        }
        const asyncValidatorArray = [];
        for (const key in asyncValidations) {
            if (asyncValidations.hasOwnProperty(key) && forms_1.Validators.hasOwnProperty(key)) {
                asyncValidatorArray.push(forms_1.Validators[key](asyncValidations[key]));
            }
        }
        return asyncValidatorArray.length > 0 ? forms_1.Validators.composeAsync(asyncValidatorArray) : null;
    }
};
exports.FormServiceDynamic = FormServiceDynamic = __decorate([
    (0, core_1.Injectable)({
        providedIn: 'root'
    })
], FormServiceDynamic);
